/*
 * Copyright 2013-2019 Software Radio Systems Limited
 *
 * This file is part of srsLTE.
 *
 * srsLTE is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of
 * the License, or (at your option) any later version.
 *
 * srsLTE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * A copy of the GNU Affero General Public License can be found in
 * the LICENSE file in the top-level directory of this distribution
 * and at http://www.gnu.org/licenses/.
 *
 */

#include <stdint.h>

// Phi values for M_sc=12 Table 5.5.1.2-1 in 36.211
int phi_M_sc_12[30][12] = {{-1, 1, 3,-3, 3, 3, 1, 1, 3, 1,-3, 3},
                                 { 1, 1, 3, 3, 3,-1, 1,-3,-3, 1,-3, 3},
                                 { 1, 1,-3,-3,-3,-1,-3,-3, 1,-3, 1,-1},
                                 {-1, 1, 1, 1, 1,-1,-3,-3, 1,-3, 3,-1},
                                 {-1, 3, 1,-1, 1,-1,-3,-1, 1,-1, 1, 3},
                                 { 1,-3, 3,-1,-1, 1, 1,-1,-1, 3,-3, 1},
                                 {-1, 3,-3,-3,-3, 3, 1,-1, 3, 3,-3, 1},
                                 {-3,-1,-1,-1, 1,-3, 3,-1, 1,-3, 3, 1},
                                 { 1,-3, 3, 1,-1,-1,-1, 1, 1, 3,-1, 1},
                                 { 1,-3,-1, 3, 3,-1,-3, 1, 1, 1, 1, 1},
                                 {-1, 3,-1, 1, 1,-3,-3,-1,-3,-3, 3,-1},
                                 { 3, 1,-1,-1, 3, 3,-3, 1, 3, 1, 3, 3},
                                 { 1,-3, 1, 1,-3, 1, 1, 1,-3,-3,-3, 1},
                                 { 3, 3,-3, 3,-3, 1, 1, 3,-1,-3, 3, 3},
                                 {-3, 1,-1,-3,-1, 3, 1, 3, 3, 3,-1, 1},
                                 { 3,-1, 1,-3,-1,-1, 1, 1, 3, 1,-1,-3},
                                 { 1, 3, 1,-1, 1, 3, 3, 3,-1,-1, 3,-1},
                                 {-3, 1, 1, 3,-3, 3,-3,-3, 3, 1, 3,-1},
                                 {-3, 3, 1, 1,-3, 1,-3,-3,-1,-1, 1,-3},
                                 {-1, 3, 1, 3, 1,-1,-1, 3,-3,-1,-3,-1},
                                 {-1,-3, 1, 1, 1, 1, 3, 1,-1, 1,-3,-1},
                                 {-1, 3,-1, 1,-3,-3,-3,-3,-3, 1,-1,-3},
                                 { 1, 1,-3,-3,-3,-3,-1, 3,-3, 1,-3, 3},
                                 { 1, 1,-1,-3,-1,-3, 1,-1, 1, 3,-1, 1},
                                 { 1, 1, 3, 1, 3, 3,-1, 1,-1,-3,-3, 1},
                                 { 1,-3, 3, 3, 1, 3, 3, 1,-3,-1,-1, 3},
                                 { 1, 3,-3,-3, 3,-3, 1,-1,-1, 3,-1,-3},
                                 {-3,-1,-3,-1,-3, 3, 1,-1, 1, 3,-3,-3},
                                 {-1, 3,-3, 3,-1, 3, 3,-3, 3, 3,-1,-1},
                                 { 3,-3,-3,-1,-1,-3,-1, 3,-3, 3, 1,-1}};

// Phi values for M_sc=24 Table 5.5.1.2-2 in 36.211
int phi_M_sc_24[30][24] = {{-1, 3, 1,-3, 3,-1, 1, 3,-3, 3, 1, 3,-3, 3, 1, 1,-1, 1, 3,-3, 3,-3,-1,-3},
                                 {-3, 3,-3,-3,-3, 1,-3,-3, 3,-1, 1, 1, 1, 3, 1,-1, 3,-3,-3, 1, 3, 1, 1,-3},
                                 { 3,-1, 3, 3, 1, 1,-3, 3, 3, 3, 3, 1,-1, 3,-1, 1, 1,-1,-3,-1,-1, 1, 3, 3},
                                 {-1,-3, 1, 1, 3,-3, 1, 1,-3,-1,-1, 1, 3, 1, 3, 1,-1, 3, 1, 1,-3,-1,-3,-1},
                                 {-1,-1,-1,-3,-3,-1, 1, 1, 3, 3,-1, 3,-1, 1,-1,-3, 1,-1,-3,-3, 1,-3,-1,-1},
                                 {-3, 1, 1, 3,-1, 1, 3, 1,-3, 1,-3, 1, 1,-1,-1, 3,-1,-3, 3,-3,-3,-3, 1, 1},
                                 { 1, 1,-1,-1, 3,-3,-3, 3,-3, 1,-1,-1, 1,-1, 1, 1,-1,-3,-1, 1,-1, 3,-1,-3},
                                 {-3, 3, 3,-1,-1,-3,-1, 3, 1, 3, 1, 3, 1, 1,-1, 3, 1,-1, 1, 3,-3,-1,-1, 1},
                                 {-3, 1, 3,-3, 1,-1,-3, 3,-3, 3,-1,-1,-1,-1, 1,-3,-3,-3, 1,-3,-3,-3, 1,-3},
                                 { 1, 1,-3, 3, 3,-1,-3,-1, 3,-3, 3, 3, 3,-1, 1, 1,-3, 1,-1, 1, 1,-3, 1, 1},
                                 {-1, 1,-3,-3, 3,-1, 3,-1,-1,-3,-3,-3,-1,-3,-3, 1,-1, 1, 3, 3,-1, 1,-1, 3},
                                 { 1, 3, 3,-3,-3, 1, 3, 1,-1,-3,-3,-3, 3, 3,-3, 3, 3,-1,-3, 3,-1, 1,-3, 1},
                                 { 1, 3, 3, 1, 1, 1,-1,-1, 1,-3, 3,-1, 1, 1,-3, 3, 3,-1,-3, 3,-3,-1,-3,-1},
                                 { 3,-1,-1,-1,-1,-3,-1, 3, 3, 1,-1, 1, 3, 3, 3,-1, 1, 1,-3, 1, 3,-1,-3, 3},
                                 {-3,-3, 3, 1, 3, 1,-3, 3, 1, 3, 1, 1, 3, 3,-1,-1,-3, 1,-3,-1, 3, 1, 1, 3},
                                 {-1,-1, 1,-3, 1, 3,-3, 1,-1,-3,-1, 3, 1, 3, 1,-1,-3,-3,-1,-1,-3,-3,-3,-1},
                                 {-1,-3, 3,-1,-1,-1,-1, 1, 1,-3, 3, 1, 3, 3, 1,-1, 1,-3, 1,-3, 1, 1,-3,-1},
                                 { 1, 3,-1, 3, 3,-1,-3, 1,-1,-3, 3, 3, 3,-1, 1, 1, 3,-1,-3,-1, 3,-1,-1,-1},
                                 { 1, 1, 1, 1, 1,-1, 3,-1,-3, 1, 1, 3,-3, 1,-3,-1, 1, 1,-3,-3, 3, 1, 1,-3},
                                 { 1, 3, 3, 1,-1,-3, 3,-1, 3, 3, 3,-3, 1,-1, 1,-1,-3,-1, 1, 3,-1, 3,-3,-3},
                                 {-1,-3, 3,-3,-3,-3,-1,-1,-3,-1,-3, 3, 1, 3,-3,-1, 3,-1, 1,-1, 3,-3, 1,-1},
                                 {-3,-3, 1, 1,-1, 1,-1, 1,-1, 3, 1,-3,-1, 1,-1, 1,-1,-1, 3, 3,-3,-1, 1,-3},
                                 {-3,-1,-3, 3, 1,-1,-3,-1,-3,-3, 3,-3, 3,-3,-1, 1, 3, 1,-3, 1, 3, 3,-1,-3},
                                 {-1,-1,-1,-1, 3, 3, 3, 1, 3, 3,-3, 1, 3,-1, 3,-1, 3, 3,-3, 3, 1,-1, 3, 3},
                                 { 1,-1, 3, 3,-1,-3, 3,-3,-1,-1, 3,-1, 3,-1,-1, 1, 1, 1, 1,-1,-1,-3,-1, 3},
                                 { 1,-1, 1,-1, 3,-1, 3, 1, 1,-1,-1,-3, 1, 1,-3, 1, 3,-3, 1, 1,-3,-3,-1,-1},
                                 {-3,-1, 1, 3, 1, 1,-3,-1,-1,-3, 3,-3, 3, 1,-3, 3,-3, 1,-1, 1,-3, 1, 1, 1},
                                 {-1,-3, 3, 3, 1, 1, 3,-1,-3,-1,-1,-1, 3, 1,-3,-3,-1, 3,-3,-1,-3,-1,-3,-1},
                                 {-1,-3,-1,-1, 1,-3,-1,-1, 1,-1,-3, 1, 1,-3, 1,-3,-3, 3, 1, 1,-1, 3,-1,-1},
                                 { 1, 1,-1,-1,-3,-1, 3,-1, 3,-1, 1, 3, 1,-1, 3, 1, 3,-3,-3, 1,-1,-1, 1, 3}};

// Prime numbers used for Section 5.5.1.1 of 36.211
#define NOF_PRIME_NUMBERS 196
uint32_t prime_numbers[NOF_PRIME_NUMBERS] = {  2,   3,   5,   7,  11,  13,  17,  19,  23,  29,
                                              31,  37,  41,  43,  47,  53,  59,  61,  67,  71,
                                              73,  79,  83,  89,  97, 101, 103, 107, 109, 113,
                                              127, 131, 137, 139, 149, 151, 157, 163, 167, 173,
                                              179, 181, 191, 193, 197, 199, 211, 223, 227, 229,
                                              233, 239, 241, 251, 257, 263, 269, 271, 277, 281,
                                              283, 293, 307, 311, 313, 317, 331, 337, 347, 349,
                                              353, 359, 367, 373, 379, 383, 389, 397, 401, 409,
                                              419, 421, 431, 433, 439, 443, 449, 457, 461, 463,
                                              467, 479, 487, 491, 499, 503, 509, 521, 523, 541,
                                              547, 557, 563, 569, 571, 577, 587, 593, 599, 601,
                                              607, 613, 617, 619, 631, 641, 643, 647, 653, 659,
                                              661, 673, 677, 683, 691, 701, 709, 719, 727, 733,
                                              739, 743, 751, 757, 761, 769, 773, 787, 797, 809,
                                              811, 821, 823, 827, 829, 839, 853, 857, 859, 863,
                                              877, 881, 883, 887, 907, 911, 919, 929, 937, 941,
                                              947, 953, 967, 971, 977, 983, 991, 997,1009,1013,
                                              1019,1021,1031,1033,1039,1049,1051,1061,1063,1069,
                                              1087,1091,1093,1097,1103,1109,1117,1123,1129,1151,
                                              1153,1163,1171,1181,1187,1193};

